<?php

namespace Drupal\responsivewrappers\Plugin\Filter;

use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * @Filter(
 *   id = "filter_bootstrap_responsive_wrapper",
 *   title = @Translation("Bootstrap Responsive Wrapper Filter"),
 *   description = @Translation("Add bootstrap responsive wrapper elements"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE
 * )
 */
class FilterBootstrapResponsiveWrapper extends FilterBase {

  public function process($text, $langcode) {
    $result = new FilterProcessResult($text);

    if ($this->settings['responsive_iframe'] || $this->settings['responsive_table'] || $this->settings['responsive_image']) {

      $dom = Html::load($text);

      // Add responsive wrapper for iframe videos.
      if ($this->settings['responsive_iframe']) {

        $iframes = $dom->getElementsByTagName('iframe');
        $video_pattern = $this->settings['responsive_iframe_pattern'];

        foreach ($iframes AS $iframe) {
          $parent_class_div = ($iframe->parentNode->tagName === 'div') ? $iframe->parentNode->getAttribute('class') : '';
          // Only add the wrapper if not exists.
          if (empty($parent_class_div) || strpos($parent_class_div, 'embed-responsive') !== FALSE) {
            $src = $iframe->getAttribute('src');
            if(preg_match($video_pattern, $src)){
              $video_wrapper = $dom->createElement('div');
              $video_wrapper->setAttribute('class', 'embed-responsive embed-responsive-16by9');
              $iframe = $iframe->parentNode->replaceChild($video_wrapper, $iframe);
              $video_wrapper->appendChild($iframe);
            }
          }
        }
      }

      // Add responsive table wrapper.
      if ($this->settings['responsive_table']) {
        $tables = $dom->getElementsByTagName('table');
        foreach ($tables AS $table) {
          // Only add the class if not exists.
          $classes = $table->getAttribute('class');
          if (!in_array($classes, ['table-responsive'])) {
            $classes = (strlen($classes) > 0) ? explode(' ', $classes) : [];
            $classes[] = 'table-responsive';
            $table->setAttribute('class', implode(' ', $classes));
          }
        }
      }

      // Add responsive image class.
      if ($this->settings['responsive_image']) {
        $images = $dom->getElementsByTagName('img');
        foreach ($images AS $image) {
          // Only add the class if not exists.
          $classes = $image->getAttribute('class');
          if (!in_array($classes, ['img-responsive'])) {
            $classes = (strlen($classes) > 0) ? explode(' ', $classes) : [];
            $classes[] = 'img-responsive';
            $image->setAttribute('class', implode(' ', $classes));
          }
        }
      }

      $result->setProcessedText(Html::serialize($dom));
    }

    return $result;
  }

  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['responsive_iframe'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Responsive video'),
      '#default_value' => isset($this->settings['responsive_iframe']) ? $this->settings['responsive_iframe'] : '',
      '#description' => $this->t('Add responsive wrapper for videos (16/9 aspect ratio).'),
    ];
    $form['responsive_iframe_pattern'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Source video pattern detection'),
      '#default_value' => isset($this->settings['responsive_iframe_pattern']) ? $this->settings['responsive_iframe_pattern'] : '#.*(youtube.|vimeo.).*#ui',
      '#description' => $this->t('Regular expresion for source video detection. This pattern evaluates scr iframe attribute.'),
    ];
    $form['responsive_table'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Responsive tables'),
      '#default_value' => isset($this->settings['responsive_table']) ? $this->settings['responsive_table'] : '',
      '#description' => $this->t('Add responsive wrapper for tables.'),
    ];
    $form['responsive_image'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Responsive images'),
      '#default_value' => isset($this->settings['responsive_image']) ? $this->settings['responsive_image'] : '',
      '#description' => $this->t('Add responsive class for images.'),
    ];

    return $form;
  }

}
